@@ -23,6 +23,7 @@ from solfege import mpd
 from solfege import statistics, statisticsviewer
 from solfege import utils
 from solfege.multipleintervalconfigwidget import MultipleIntervalConfigWidget
+from solfege.mpd import elems
 
 import solfege
 
@@ -41,7 +42,12 @@ class Teacher(abstract.MelodicIntervalTeacher):
         """
         Return a string that can be used to display the intervals.
         """
-        s = self.m_tonika.get_octave_notename()
+        score = elems.Score()
+        staff = score.add_staff()
+        staff.set_property(elems.Rat(0, 1), 'hide-barline', True)
+        staff.set_property(elems.Rat(0, 1), 'hide-timesignature', True)
+        score.voice11.append(elems.Note(self.m_tonika,
+            solfege.mpd.Duration(4, 0)))
         last = self.m_tonika
         for i in self.m_question:
             n = mpd.Interval()
@@ -49,8 +55,9 @@ class Teacher(abstract.MelodicIntervalTeacher):
             last = last + n
             if abs(last.m_accidental_i) > 1:
                 last.normalize_double_accidental()
-            s = s + " " + last.get_octave_notename()
-        return self.m_tonika, r"\staff{ \clef %s %s}" % (mpd.select_clef(s), s)
+            score.voice11.append(elems.Note(last,
+                solfege.mpd.Duration(4, 0)))
+        return score
     def guessed_correct(self):
         if self.get_int('number_of_intervals=1') == 1 \
                 and not self.m_custom_mode:
@@ -125,9 +132,7 @@ class Gui(abstract.Gui):
                                   self.get_string('user/highest_pitch'))
         if r == Teacher.OK:
             self.m_t.play_first_tone()
-            q = self.m_t.get_question()
-            fontsize = self.get_int('config/feta_font_size=20')
-            self.g_score_displayer.display(q[1], fontsize)
+            self.g_score_displayer.display_score(self.m_t.get_question())
             self.g_new_interval_wrong.set_sensitive(True)
             self.g_repeat_tonika.set_sensitive(True)
             self.g_play_answer.set_sensitive(True)
@@ -155,6 +160,9 @@ class Gui(abstract.Gui):
                 else:
                     break
             self.set_int('number_of_intervals', i)
+        # This exercise will be in expert mode when the lesson file
+        # does not specify which questions to ask. It will ignore
+        # the setting in the preferences window
         self.m_t.m_custom_mode = bool(
                       not self.m_t.m_P.header.ask_for_intervals_0)
         super(Gui, self).on_start_practise()
